/* securitytoken.h - api for reading and writing data to a security token 
 *
 * Copyright (C) 2006 Ray Strode
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */
#ifndef SC_SECURITY_TOKEN_H
#define SC_SECURITY_TOKEN_H

#include <glib.h>
#include <glib-object.h>

#include <secmod.h>

G_BEGIN_DECLS
#define SC_TYPE_SECURITY_TOKEN            (sc_security_token_get_type ())
#define SC_SECURITY_TOKEN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SC_TYPE_SECURITY_TOKEN, ScSecurityToken))
#define SC_SECURITY_TOKEN_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SC_TYPE_SECURITY_TOKEN, ScSecurityTokenClass))
#define SC_IS_SECURITY_TOKEN(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SC_TYPE_SECURITY_TOKEN))
#define SC_IS_SECURITY_TOKEN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SC_TYPE_SECURITY_TOKEN))
#define SC_SECURITY_TOKEN_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), SC_TYPE_SECURITY_TOKEN, ScSecurityTokenClass))
#define SC_SECURITY_TOKEN_ERROR           (sc_security_token_error_quark ())
typedef struct _ScSecurityTokenClass ScSecurityTokenClass;
typedef struct _ScSecurityToken ScSecurityToken;
typedef struct _ScSecurityTokenPrivate ScSecurityTokenPrivate;
typedef enum _ScSecurityTokenError ScSecurityTokenError;
typedef enum _ScSecurityTokenState ScSecurityTokenState;

typedef struct _ScSecurityTokenRequest ScSecurityTokenRequest;

struct _ScSecurityToken {
    GObject parent;

    /*< private > */
    ScSecurityTokenPrivate *priv;
};

struct _ScSecurityTokenClass {
    GObjectClass parent_class;

    void (* inserted) (ScSecurityToken *token);
    void (* removed) (ScSecurityToken *token);
};

enum _ScSecurityTokenError {
    SC_SECURITY_TOKEN_ERROR_GENERIC = 0,
};

enum _ScSecurityTokenState {
    SC_SECURITY_TOKEN_STATE_INSERTED = 0,
    SC_SECURITY_TOKEN_STATE_REMOVED,
};

GType sc_security_token_get_type (void) G_GNUC_CONST;
GQuark sc_security_token_error_quark (void) G_GNUC_CONST;

CK_SLOT_ID sc_security_token_get_slot_id (ScSecurityToken *token);
gint sc_security_token_get_slot_series (ScSecurityToken *token);
ScSecurityTokenState sc_security_token_get_state (ScSecurityToken *token);

gchar *sc_security_token_get_name (ScSecurityToken *token);
gboolean sc_security_token_is_login_token (ScSecurityToken *token);

gboolean sc_security_token_unlock (ScSecurityToken *token,
				   const gchar     *password);

/* don't under any circumstances call these functions */
#ifdef SC_SECURITY_TOKEN_ENABLE_INTERNAL_API

ScSecurityToken *_sc_security_token_new (SECMODModule *module,
					 CK_SLOT_ID slot_id, 
					 gint slot_series);
ScSecurityToken *_sc_security_token_new_from_name (SECMODModule *module,
						   const gchar *name);

void _sc_security_token_set_state (ScSecurityToken      *token,
				   ScSecurityTokenState  state);
#endif 

G_END_DECLS
#endif				/* SC_SECURITY_TOKEN_H */
