/* securitytokenmonitor.h - monitor for security token insertion and
 *                          removal events
 *
 * Copyright (C) 2006 Ray Strode
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */
#ifndef SC_SECURITY_TOKEN_MONITOR_H
#define SC_SECURITY_TOKEN_MONITOR_H

#define SC_SECURITY_TOKEN_ENABLE_INTERNAL_API
#include "securitytoken.h"

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS
#define SC_TYPE_SECURITY_TOKEN_MONITOR            (sc_security_token_monitor_get_type ())
#define SC_SECURITY_TOKEN_MONITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SC_TYPE_SECURITY_TOKEN_MONITOR, ScSecurityTokenMonitor))
#define SC_SECURITY_TOKEN_MONITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SC_TYPE_SECURITY_TOKEN_MONITOR, ScSecurityTokenMonitorClass))
#define SC_IS_SECURITY_TOKEN_MONITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SC_TYPE_SECURITY_TOKEN_MONITOR))
#define SC_IS_SECURITY_TOKEN_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SC_TYPE_SECURITY_TOKEN_MONITOR))
#define SC_SECURITY_TOKEN_MONITOR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), SC_TYPE_SECURITY_TOKEN_MONITOR, ScSecurityTokenMonitorClass))
#define SC_SECURITY_TOKEN_MONITOR_ERROR           (sc_security_token_monitor_error_quark ())
typedef struct _ScSecurityTokenMonitor ScSecurityTokenMonitor;
typedef struct _ScSecurityTokenMonitorClass ScSecurityTokenMonitorClass;
typedef struct _ScSecurityTokenMonitorPrivate ScSecurityTokenMonitorPrivate;
typedef enum _ScSecurityTokenMonitorError ScSecurityTokenMonitorError;

struct _ScSecurityTokenMonitor {
    GObject parent;

    /*< private > */
    ScSecurityTokenMonitorPrivate *priv;
};

struct _ScSecurityTokenMonitorClass {
    GObjectClass parent_class;

    /* Signals */
    void (*security_token_inserted) (ScSecurityTokenMonitor *monitor,
				     ScSecurityToken *token);
    void (*security_token_removed) (ScSecurityTokenMonitor *monitor,
				    ScSecurityToken *token);
    void (*error) (ScSecurityTokenMonitor *monitor, 
		   GError                 *error);
};

enum _ScSecurityTokenMonitorError {
    SC_SECURITY_TOKEN_MONITOR_ERROR_GENERIC = 0,
    SC_SECURITY_TOKEN_MONITOR_ERROR_WITH_NSS,
    SC_SECURITY_TOKEN_MONITOR_ERROR_LOADING_DRIVER,
    SC_SECURITY_TOKEN_MONITOR_ERROR_WATCHING_FOR_EVENTS,
    SC_SECURITY_TOKEN_MONITOR_ERROR_REPORTING_EVENTS
};

GType sc_security_token_monitor_get_type (void) G_GNUC_CONST;
GQuark sc_security_token_monitor_error_quark (void) G_GNUC_CONST;

ScSecurityTokenMonitor *sc_security_token_monitor_new (const gchar *module);

gboolean sc_security_token_monitor_start (ScSecurityTokenMonitor  *monitor, 
				 	  GError                 **error);

void sc_security_token_monitor_stop (ScSecurityTokenMonitor *monitor);

gchar *sc_security_token_monitor_get_module_path (ScSecurityTokenMonitor *monitor);
gboolean sc_security_token_monitor_login_token_is_inserted (ScSecurityTokenMonitor *monitor);

G_END_DECLS
#endif				/* SC_SECURITY_TOKEN_MONITOR_H */
